/*
  Name: AI.cpp
  Copyright: Polecatsoft.tk
  Author: Deathlord
  Date: 18.06.05 05:30
  Description: Enemy AI behaviour
*/
#include "StdAfx.h"
#include "unit.h"
#include "AI.h"

enum actions
{
	n = -1,
	a = 0,
	d = 1,
	h = 2,
	s = 3
};

enum AI_PAT_TYPE
{
	PAT_AGRESSIVE = 0,
	PAT_NEUTRAL = 1,
	PAT_DEFEND = 2
};

const int S_SPRING = 10;
const int S_SUMMER = 20;
const int S_FALL = 15;
const int S_WINTER = 20;

//[subPat = which of the 20 subPattern to take]
//[was used for determining agressive, neutral, defensive, now used to distinguish chartype]
//[values = action 1, mana 1 etc.]
//[][chartype: spring, summer, fall, winter][action1, mana1, action2, mana2, action3, mana3]
const int AI_PAT_AGRESSIVE[20][4][6] =
{
	{ {a, 7, a, 10, a , 3}, {s, S_SUMMER, n, 0, n, 0}, {a, 10, a, 5, a, 5}, {a, 10, a, 5, a, 10} },
	{ {s, S_SPRING, a, 5, a, 5}, {a, 10, a, 5, d, 5}, {a, 7, d, 3, a, 10}, {a, 7, a, 7, d, 6} },
	{ {s, S_SPRING, d, 5, a, 5}, {d, 5, a, 12, a, 3}, {d, 4, a, 6, a, 10}, {a, 7, a, 6, a, 7} },
	{ {a, 10, a, 6, h, 4}, {h, 6, a, 6, a, 8}, {d, 4, a, 10, h, 6}, {d, 5, a, 10, a, 5} },
	{ {a, 7, a, 6, a, 7}, {a, 12, a, 4, a, 4}, {s, S_FALL, a, 5, n, 0}, {s, S_WINTER, n, 0, n, 0} },
	{ {s, S_SPRING, a, 7, a, 3}, {d, 6, a, 12, a, 2}, {a, 5, s, S_FALL}, {a, 10, d, 5, d, 5} },
	{ {a, 10, a, 4, d, 6}, {a, 10, d, 4, a, 6}, {a, 12, d, 4, h, 4}, {a, 16, h, 4, n, 0} },
	{ {a, 16, h, 4, n, 0}, {a, 14, h, 6, n, 0}, {d, 4, a, 16, n, 0}, {s, S_WINTER, n, 0, n, 0} },
	{ {a, 10, d, 5, a, 5}, {a, 12, d, 4, a, 4}, {s, S_FALL, a, 5, n, 0}, {a, 17, d, 3, n, 0} },
	{ {s, S_SPRING, n, 0, a, 10}, {d, 6, a, 12, h, 4}, {a, 16, d, 2, a, 2}, {a, 10, d, 4, h, 6} },
	{ {d, 4, a, 13, d, 3}, {s, S_SUMMER, n, 0, n, 0}, {d, 4, a, 10, h, 6}, {a, 7, a, 7, h, 6} },
	{ {a, 15, n, 0, a, 5}, {a, 7, a, 8, d, 5}, {h, 4, a, 8, a, 8}, {s, S_WINTER, n, 0, n, 0} },
	{ {a, 6, a, 7, a, 6}, {a, 6, a, 6, a, 8}, {a, 10, d, 5, a, 5}, {d, 5, a, 10, a, 5} },
	{ {a, 12, h, 4, a, 4}, {d, 5, a, 10, a, 5}, {d, 5, a, 8, a, 7}, {a, 20, n, 0, n, 0} },
	{ {a, 20, n, 0, n, 0}, {a, 10, h, 10, n, 0}, {h, 10, a, 5, a, 5}, {a, 15, a, 5, n, 0} },
	{ {n, 0, a, 14, h, 6}, {d, 8, a, 8, a, 4}, {a, 9, a, 9, h, 2}, {d, 3, a, 15, h, 2} },
	{ {a, 15, d, 5, n, 0}, {a, 15, d, 5, n, 0}, {a, 7, d, 7, a, 6}, {a, 10, a, 10, n, 0} },
	{ {a, 7, a, 7, a, 6}, {a, 20, n, 0, n, 0}, {a, 20, n, 0, n, 0}, {a, 17, d, 1, h, 2} },
	{ {a, 15, a, 3, a, 2}, {d, 4, a, 12, d, 4}, {a, 15, a, 5, n, 0}, {s, S_WINTER, n, 0, 0} },
	{ {s, S_SPRING, a, 10, n, 0}, {a, 20, n, 0, n, 0}, {a, 18, d, 1, d, 1}, {a, 16, a, 2, a, 2} },
};

const int AI_PAT_NEUTRAL[20][4][6] =
{
	{ {d, 5, a, 10, a, 5}, {s, S_SUMMER, n, 0, n, 0}, {a, 7, d, 7, h, 6}, {a, 10, d, 5, d, 5} },
	{ {s, S_SPRING, a, 5, d, 5}, {a, 10, d, 4, h, 6}, {a, 5, s, S_FALL, n, 0}, {d, 5, h, 10, a, 5} },
	{ {d, 7, a, 7, h, 6}, {h, 10, a, 10, n, 0}, {a, 5, h, 10, d, 5}, {a, 7, d, 3, h, 10} },
	{ {h, 10, d, 5, a, 5}, {s, S_SUMMER, n, 0, n, 0}, {a, 5, d, 5, h, 10}, {d, 5, d, 5, a, 10} },
	{ {d, 7, a, 7, h, 6}, {d, 5, d, 5, a, 10}, {d, 5, d, 5, a, 10}, {a, 7, d, 7, d, 6} },
	{ {a, 10, d, 7, d, 3}, {a, 10, a, 5, d, 5}, {a, 5, s, S_FALL, n, 0}, {h, 10, a, 6, a, 4} },
	{ {h, 12, a, 4, a, 4}, {d, 4, h, 4, a, 12}, {d, 5, a, 10, d, 5}, {d, 7, d, 7, d, 6} },
	{ {a, 9, d, 5, h, 6}, {a, 6, a, 6, h, 8}, {a, 7, a, 7, d, 6}, {d, 7, a, 7, a, 6} },
	{ {h, 10, a, 5, a, 5}, {a, 12, d, 4, d, 4}, {h, 6, d, 6, a, 8}, {d, 5, h, 10, a, 5} },
	{ {s, S_SPRING, a, 10, n, 0}, {s, S_SUMMER, n, 0, n, 0}, {d, 3, a, 2, s, S_FALL}, {a, 10, a, 5, d, 5} },
	{ {d, 6, h, 14, n, 0}, {s, S_SUMMER, n, 0, n, 0}, {d, 7, a, 7, d, 6}, {d, 5, h, 8, a, 7} },
	{ {a, 7, a, 8, d, 5}, {a, 7, d, 7, h, 6}, {a, 10, d, 5, d, 5}, {d, 6, d, 9, a, 5} },
	{ {h, 10, a, 5, a, 5}, {h, 6, a, 8, h, 6}, {h, 10, d, 5, a, 5}, {h, 8, a, 11, d, 1} },
	{ {h, 10, a, 10, n, 0}, {a, 15, d, 5, n, 0}, {a, 7, d, 7, h, 6}, {s, S_WINTER, n, 0, n, 0} },
	{ {n, 0, a, 15, d, 5}, {s, S_SUMMER, n, 0, n, 0}, {a, 5, s, S_FALL, n, 0}, {a, 7, d, 7, d, 6} },
	{ {h, 20, n, 0, n, 0}, {d, 10, h, 10, n, 0}, {a, 5, s, S_FALL, n, 0}, {d, 6, h, 10, a, 4} },
	{ {s, S_SPRING, a, 5, a, 5}, {a, 7, d, 6, a, 7}, {a, 7, d, 6, a, 7}, {a, 5, d, 10, a, 5} },
	{ {a, 10, d, 10, n, 0}, {h, 10, d, 5, a, 5}, {d, 10, a, 5, a, 5}, {d, 7, a, 7, h, 6} },
	{ {a, 10, d, 5, a, 5}, {a, 7, d, 7, a, 6}, {a, 7, h, 10, d, 3}, {a, 9, d, 4, d, 7} },
	{ {d, 7, d, 7, a, 6}, {h, 20, n, 0, n, 0}, {a, 5, s, S_FALL}, {s, S_WINTER, n , 0, n, 0} },
};

const int AI_PAT_DEFENSIVE[20][4][6] =
{
	{ {s, S_SPRING, d, 5, d, 5}, {d, 10, h, 6, h, 4}, {s, S_FALL, d, 5, n, 0}, {d, 10, d, 5, a, 5} },
	{ {d, 7, d, 7, a, 6}, {a, 6, h, 6, d, 8}, {d, 5, d, 5, d, 10}, {d, 7, d, 7, h, 6} },
	{ {d, 5, h, 10, a, 5}, {d, 10, a, 4, d, 6}, {d, 5, s, S_FALL, n, 0}, {d, 5, d, 5, a, 10} },
	{ {h, 10, h, 10, n, 0}, {h, 6, h, 6, a, 8}, {d, 5, a, 10, a, 5}, {a, 10, d, 5, a, 10} },
	{ {d, 7, a, 8, d, 4}, {d, 5, d, 10, a, 5}, {a, 6, d, 7, a, 7}, {s, S_WINTER} },
	{ {h, 8, d, 8, a, 4}, {d, 5, a, 9, h, 6}, {a, 3, d, 2, s, S_FALL}, {d, 5, h, 10, a, 5} },
	{ {s, S_SPRING, d, 5, a, 5}, {a, 5, d, 10, d, 5}, {a, 10, d, 5, d, 5}, {d, 4, d, 4, a, 12} },
	{ {s, S_SPRING, a, 10, n, 0}, {d, 7, h, 6, }, {a, 7, d, 7, h, 6}, {d, 10, h, 10, n, 0} },
	{ {d, 3, d, 3, h, 14}, {d, 10, h, 10, n, 0}, {a, 5, s, S_FALL}, {h, 20, n, 0, n, 0} },
	{ {h, 10, d, 3, a, 7}, {d, 6, a, 10, h, 4}, {d, 7, a, 7, d, 6}, {h, 16, a, 4, n, 0} },
	{ {d, 7, d, 7, d, 6}, {d, 6, d, 6, d, 8}, {h, 6, a, 7, d, 7}, {d, 6, d, 6, a, 8} },
	{ {d, 6, d, 8, d, 6}, {s, S_SUMMER, n, 0, n, 0}, {a, 5, s, S_FALL}, {s, S_WINTER, n, 0, n, 0} },
	{ {h, 8, a, 8, d, 4}, {d, 10, a, 4, h, 6}, {d, 7, d, 7, a, 6}, {d, 5, d, 5, a, 10} },
	{ {d, 20, n, 0, n, 0}, {d, 6, d, 6, h, 8}, {d, 5, d, 5, a, 10}, {h, 20, n, 0, n, 0} },
	{ {h, 10, S_SPRING, n, 0}, {d, 5, h, 10, a, 5}, {s, S_FALL, a, 5, n, 0}, {a, 20, n, 0, n, 0} },
	{ {d, 10, a, 5, a, 5}, {d, 20, n, 0, n, 0}, {h, 6, a, 4, d, 10},  {d, 10, d, 5, a, 5}},
	{ {S_SPRING, n, 0, a, 10}, {a, 9, d, 5, d, 6}, {d, 20, n, 0, n, 0}, {d, 10, d, 10, n, 0} },
	{ {d, 15, n, 0, a, 5}, {d, 3, h, 16, a, 1}, {d, 5, d, 5, d, 10}, {a, 5, d, 5, h, 10} },
	{ {h, 6, h, 8, a, 6}, {d, 4, d, 4, h, 12}, {d, 10, a, 10, n, 0}, {a, 15, d, 3, d, 2} },
	{ {h, 8, a, 8, d, 4}, {a, 10, d, 10, n, 0}, {d, 15, a, 3, a, 2}, {s, S_WINTER, n, 0, n, 0} },
};

/*AI behaviour by chartype
first []: chartype; second []: border values, when to act defensive,
agressive or neutral for specific chartype
[][0 + 1] -> agressive from-to
[][2 + 3] -> neutral from-to
[][4 + 5] -> devensive from-to
[][6] -> added to the the random value on the used pattern (if overflow, random value is substracted)
*/
const int AI_BEHAVIOUR[4][7] = {
	{20, 0, 100, 50, 49, 21, 3}, //spring
	{30, 0, 100, 60, 59, 31, 1}, //summer
	{100, 80, 79, 30, 29, 0, 0}, //fall
	{100, 45, 44, 21, 20, 0, 5} //winter
};

void applyKI(fightUnit *u, int mainPat, int subPat, int behav)
{
	switch(mainPat)
	{
		case PAT_AGRESSIVE:
			{
				u->attack[0] = AI_PAT_AGRESSIVE[subPat][behav][0];
				u->attackMana[0] = AI_PAT_AGRESSIVE[subPat][behav][1];
				u->attack[1] = AI_PAT_AGRESSIVE[subPat][behav][2];
				u->attackMana[1] = AI_PAT_AGRESSIVE[subPat][behav][3];
				u->attack[2] = AI_PAT_AGRESSIVE[subPat][behav][4];
				u->attackMana[2] = AI_PAT_AGRESSIVE[subPat][behav][5];
				break;
			}
		case PAT_NEUTRAL:
			{
				u->attack[0] = AI_PAT_NEUTRAL[subPat][behav][0];
				u->attackMana[0] = AI_PAT_NEUTRAL[subPat][behav][1];
				u->attack[1] = AI_PAT_NEUTRAL[subPat][behav][2];
				u->attackMana[1] = AI_PAT_NEUTRAL[subPat][behav][3];
				u->attack[2] = AI_PAT_NEUTRAL[subPat][behav][4];
				u->attackMana[2] = AI_PAT_NEUTRAL[subPat][behav][5];
				break;
			}
		case PAT_DEFEND:
			{
				u->attack[0] = AI_PAT_DEFENSIVE[subPat][behav][0];
				u->attackMana[0] = AI_PAT_DEFENSIVE[subPat][behav][1];
				u->attack[1] = AI_PAT_DEFENSIVE[subPat][behav][2];
				u->attackMana[1] = AI_PAT_DEFENSIVE[subPat][behav][3];
				u->attack[2] = AI_PAT_DEFENSIVE[subPat][behav][4];
				u->attackMana[2] = AI_PAT_DEFENSIVE[subPat][behav][5];
				break;
			}
	}
	int myHP = u->life;
	//make sure there's no healing if it isn't neccessary
	for(int i = 0; i <= 2; ++i)
	{
		if(u->attack[i] == h)
		{
			if((u->attackMana[i] / 2) + myHP > 100)
			{
				u->attack[i] = a;
			}
		}
		if(u->attack[i] == s && behav == 2 && myHP + 15 > 100) //fall: special heal case
		{
			u->attack[i] = a;
		}
	}

	//make sure there's healing if it's a really lowHP case
	if(myHP <= 15)
	{
		int has_healing = 0;
		for(int i = 0; i <= 2; ++i)
		{
			if(u->attack[i] == h)
			{
				has_healing += u->attackMana[i];
			}
		}
		
		if(has_healing <= 8)
		{
			if(behav != 2) //not for fall
			{
				u->attack[0] = h;
				u->attackMana[0] = 10;
				if(u->attack[1] == h)
				{
					u->attack[1] = a;
				}
				if(u->attack[2] == h)
				{
					u->attack[2] = d;
				}
				u->attackMana[1] = 5;
				u->attackMana[2] = 5;
			}
			else //fall
			{
				u->attack[0] = s;
				u->attackMana[0] = 15;
				u->attackMana[2] = 5;
				u->attack[2] = n;
				u->attackMana[2] = 0;
				int random = rand() % 10;
				if(random > 5)
				{
					u->attack[1] = a;
				}
				else
				{
					u->attack[1] = d;
				}
			}
		}
	}
	
	//fix some strange bugs with invalid mana values
	{
		for(int i = 0; i <= 2; ++i)
		{
			if(u->attackMana[i] == 0 || u->attackMana[i] > 20 && u->attack[i] != n)
			{
				u->attack[i] = n;
			}
		}
	}
}

//determine which patten to use corresponding to character's behaviour
void determinePat(fightUnit *u, int behav, int &mainPat, int &subPat)
{
    int random = rand() % 19;
    int myHP = u->life;
    int mPat;
    int sPat;
    if(myHP <= AI_BEHAVIOUR[behav][0] && myHP >= AI_BEHAVIOUR[behav][1])
        mPat = PAT_AGRESSIVE;
	if(myHP <= AI_BEHAVIOUR[behav][2] && myHP >= AI_BEHAVIOUR[behav][3])
	    mPat = PAT_NEUTRAL;
    if(myHP <= AI_BEHAVIOUR[behav][4] && myHP >= AI_BEHAVIOUR[behav][5])
        mPat = PAT_DEFEND;
    sPat = random + AI_BEHAVIOUR[behav][6];
    if(sPat > 19)
    {
		sPat -= rand() % 10;
	}
	mainPat = mPat;
	subPat = sPat;
}

 //determine character's behaviour from chartype
int determineBehav(CT chartype)
{
	switch(chartype)
	{
  	case CT_SPRING:
		{
			return 0;
			break;
		}
 	case CT_SUMMER:
		{
            return 1;
			break;
		}
  	case CT_FALL:
		{
            return 2;
			break;
		}
  	case CT_WINTER:
		{
            return 3;
			break;
		}
	}
	return -1;
}

void doAI(fightUnit &bot, CT chartype)
{
	int behav = determineBehav(chartype);
	if(behav == -1)
		error::fatal("AI error: Invalid AI behaviour.");
	int mainPat;
	int subPat;
	determinePat(&bot, behav, mainPat, subPat);
	if(mainPat == -1 || subPat == -1)
		error::fatal("AI error: Invalid AI pattern.");
	applyKI(&bot, mainPat, subPat, behav);
}
