/*
  Name: game.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 17.06.05 23:02
  Description: The core game function
*/

#include "StdAfx.h"
using namespace std;
#include "game.h"
#include "unit.h"
#include "AI.h"
#include "messageClass.h"

#include "fightImg.h"
#include "ButtonAttack.h"
#include "ButtonDefense.h"
#include "ButtonHeal.h"
#include "ButtonSpecial.h"

const string names[4] = { "Spring", "Summer",
						"Fall", "Winter" };
						
const string attackNames[4][4] = {
	{ "Typhoon",    "Wind wall", "Heal", "Confusing joint"},
	{ "Fireball",   "Magma wall", "Energy Absorption", "Vendetta"},
	{ "Earthquake", "Magic shield", "Heal", "Heal of nature"},
	{ "Icebolt",    "Magic shield", "Ice is life", "Frosty twist"}
};

const string description[16][3] = {
	{ "A typhoon will blow away the enemy", "", "Causes 1 damage for every mana point spent" },
	{ "A strong wind will reduce the damage of enemy attacks", "", "Reduces the damage of enemy attacks by 1 for each mana point spent" },
	{ "This spell will heal you", "", "Turns half of the mana spent into life" },
	{ "THC in the blood reduces the effects of all enemy spells by 50%", "Does not affect the special attack", "" },

	{ "A ball of fire will burn the enemy", "", "Causes 1 damage for every mana point spent" },
	{ "A wall made of magma will pop up and reduce the damage of enemy", "attacks", "Reduces the damage of enemy attacks by 1 for each mana point spent" },
	{ "Energy from the sun will be absorbed and turned into life", "", "Turns half of the mana spent into life" },
	{ "Casting this spell will decrease the enemy's life by 30.", "You lose 20 life yourself,", "but it's still a powerful attack" },

	{ "An earthquake will shake the enemy", "", "Causes 1 damage for every mana point spent" },
	{ "A mana shield will block attacks", "", "Reduces the damage of enemy attacks by 1 for each mana point spent" },
	{ "This spell will heal you", "", "Turns half of the mana spent into life" },
	{ "Using forces of nature and the element of earth,", "you will receive 15 health points for only 15 mana points", "" },

	{ "A bolt of ice will be shot on the enemy", "", "Causes 1 damage for every mana point spent" },
	{ "A mana shield will block attacks", "", "Reduces the damage of enemy attacks by 1 for each mana point spent" },
	{ "The cold powers of ice will increase your life", "", "Turns half of the mana spent into life" },
	{ "This is a 'no risk, no fun' spell.", "You will receive life for every attack spell the enemy casts", "and lose life for every defend spell the enemy casts" },
};

const int manaSpecial[4] = { 10, 20, 15, 20 };

void showNames(unit *weazards, int battle);
enum CURSTATE
{
	SELECTATTACKS,
	SELECTMANA,
	AISTATE,
	ANIMATE,
	RESET,
	PREPAREWON,
	WON,
	PREPARELOSE,
	LOSE,
};

int fight(unit *weazards, BITMAP *background, BITMAP *buttons[4])
{
	fightUnit units[2];
	messageClass messages(12, 12, 20, global::dbuffer);
	string versus = weazards[0].getName() + string(" versus ") + weazards[1].getName();

	int retVal;
	bool run = true;
	int frameSkip = 0, i;
	CURSTATE state = SELECTATTACKS;
	int curAttack = 0;
	int curFieldSelected = 0;
	int curMana = 0;
	int animCountdown;
	int curAnimation;
	//Colors for texts
	int color[2] = {makecol(0xCC, 0x0, 0x00), makecol(0x00, 0x00, 0xCC) };
	//Colors for spells
	int colorSpell[4] = {makecol(0x00, 0xFF, 0x00), makecol(0xFF, 0x00, 0x00),
		makecol(0x6B, 0x52, 0x0E), makecol(0x00, 0x00, 0xFF) };
	//Special attack booleans
	bool winter[2] = { false, false },
	     spring[2] = { false, false };
	messages.send(string("Beginning next match, ") + versus, 0);
	while(run)
	{
		while(timerCounter > 0)
		{
			switch(state)
			{
			case SELECTATTACKS:
				if(keyStates[KEY_UP])
				   --curFieldSelected &= 3;
				if(keyStates[KEY_DOWN])
				    ++curFieldSelected &= 3;
				if(keyStates[KEY_ENTER] || !units[0].mana)
				{
					state = SELECTMANA;
					curMana = 0;
				}
				if(keyStates[KEY_BACKSPACE])
				{
					if(curAttack)
					{
						--curAttack;
						units[0].attack[curAttack] = -1;
						units[0].mana += units[0].attackMana[curAttack];
					}
				}
				break;
			case SELECTMANA:
				if(!units[0].mana || (curFieldSelected == 3 && units[0].mana < manaSpecial[weazards[0].getType()]))
				{
					if(units[0].mana)
					{
						state = SELECTATTACKS;
						break;
					}
					state = AISTATE;
					break;
				}
				if(keyStates[KEY_ENTER])
				{
					//If the player wants to heal, use number that can be divided by 2
					if(curFieldSelected == 2)
					{
					    curMana ^= curMana & 1;
					    if(!curMana)
						{
							if(units[0].mana >= 2)
					        	curMana = 2;
							else
							    curMana = 1;
						}
					}
					units[0].attack[curAttack] = curFieldSelected;
					units[0].attackMana[curAttack] = curMana;
					units[0].mana -= curMana;
					++curAttack;
					if(curAttack == 3 || !units[0].mana)
					{
						state = AISTATE;
						break;
					}
					else
					{
						state = SELECTATTACKS;
						break;
					}
				}
				if(keyStates[KEY_BACKSPACE])
				{
					state = SELECTATTACKS;
					break;
				}
				if(key[KEY_UP])
				    ++curMana;
				if(key[KEY_DOWN])
				    --curMana;
				if(keyStates[KEY_RIGHT])
				    ++curMana;
				if(keyStates[KEY_LEFT])
				    --curMana;
				if(curFieldSelected == 3)
				{
					if(curMana != manaSpecial[weazards[0].getType()])
					    curMana = manaSpecial[weazards[0].getType()];
				}
				else
				{
					if(curMana > 20)
					    curMana = 20;
					if(curMana > units[0].mana)
					    curMana = units[0].mana;
					if(curMana < 1)
					    curMana = 1;
				}
				break;
			case AISTATE:
				state = ANIMATE;
				doAI(units[1], weazards[1].getType());
				animCountdown = 64;
				curAnimation = 0;
				for(i = 0; i < 2; ++i)
				{
			    	winter[i] = false;
	   				spring[i] = false;
				}
			case ANIMATE:
				--animCountdown;
				if(!animCountdown)
				{
					animCountdown = 64;
					int damage[2] = { 0, 0 };
					bool specialTurn = units[0].attack[curAnimation] != 3 &&
										units[1].attack[curAnimation] == 3;
					if(specialTurn)
					    i = 1;
				    else
					    i = 0;
					for( ; i < 2; ++i)
					{
						int manaUsed = units[i].attackMana[curAnimation];
						if(spring[i ^ 1])
						    manaUsed >>= 1;
						//Attack
						switch(units[i].attack[curAnimation])
						{
						case 0:
							messages.send(string("Player ") + i2str(i + 1) + string(" casts an"
							" attack spell with ") + i2str(units[i].attackMana[curAnimation])
							+ string(" damage"), color[i]);
                            if(!winter[i ^ 1])
                            {
	                            damage[i ^ 1] += manaUsed;
							}
							else
	   						{
								units[i ^ 1].life += manaUsed / 3;
								if(units[i ^ 1].life > 100)
								    units[i ^ 1].life = 100;
							}
							break;
						case 1:
							messages.send(string("Player ") + i2str(i + 1) + string(" casts a"
							" defensive spell with a strength of ") + i2str(units[i].attackMana[curAnimation]), color[i]);
                            if(!winter[i ^ 1])
								damage[i] -= manaUsed;
							else
	   						{
								units[i ^ 1].life -= manaUsed  * 3;
							}
							break;
						case 2:
							messages.send(string("Player ") + i2str(i + 1) + string(" casts a"
							" healing spell and receives ") + i2str((units[i].attackMana[curAnimation] >> 1) + (units[i].attackMana[curAnimation] & 1))
							+ string(" health points"), color[i]);
							units[i].life += manaUsed >> 1;
							if(units[i].life > 100)
							    units[i].life = 100;
						    break;
					    case 3:
							switch(weazards[i].getType())
							{
							case 0:
								messages.send("The effect of all enemy spells is cut by 50%", color[i]);
								spring[i] = true;
								break;
							case 1:
								messages.send("He deals 30 damage and loses 20 health points", color[i]);
								damage[i] += 20;
								damage[i ^ 1] += 30;
								break;
							case 2:
								messages.send("He receives 15 health points", color[i]);
								units[i].life += 15;
								if(units[i].life > 100)
								    units[i].life = 100;
								break;
							case 3:
								messages.send("All attacks will raise and all defenses will reduce its life", color[i]);
								winter[i] = true;
								break;
							};
							messages.send(string("Player ") + i2str(i + 1) + string(" uses his"
							" special skill."), color[i]);
						};
						if(specialTurn && i == 1)
						    i = -1;
						else if(specialTurn && i == 0)
							i = 1;
					}
					for(i = 0; i < 2; ++i)
					{
						if(damage[i] > 0)
						{
							messages.send(string("Player ") + i2str(i + 1)
							+ string(" loses ") + i2str(damage[i]) + string(" health points"), color[i]);
						    units[i].life -= damage[i];
						}
						if(units[i].life <= 0)
						{
							units[i].life = 0;
						    retVal = i;
						    if(retVal == 0)
						        state = PREPARELOSE;
							else
								state = PREPAREWON;
						    break;
						}
					}
					units[0].attack[curAnimation] = -1;
					if(curAnimation == 2 ||
					(units[0].attack[curAnimation + 1] == -1 && units[1].attack[curAnimation + 1] == -1))
					{
						state = RESET;
						break;
					}
					++curAnimation;
				}
				break;
			case RESET:
				curAttack = 0;
				curFieldSelected = 0;
				for(i = 0; i < 3; ++i)
					units[0].attack[i] = -1;
				units[0].mana = 20;
				state = SELECTATTACKS;
				break;
			case PREPAREWON:
				//Clear messages
				for(i = 0; i < 8; ++i)
				    messages.output();
				messages.send("You have won!", color[0]);
				animCountdown = 64;
				state = WON;
			case WON:
				--animCountdown;
				if(!animCountdown)
				{
					run = false;
				}
				break;
			case PREPARELOSE:
				//Clear messages
				for(i = 0; i < 8; ++i)
				    messages.output();
				messages.send("You have lost.", color[1]);
				animCountdown = 64;
				state = LOSE;
			case LOSE:
				--animCountdown;
				if(!animCountdown)
				{
					run = false;
				}
				break;
			};
			if(keyStates[KEY_ESC])
			{
				if(alert("Are you sure that you want to return to the main menu?", 0, 0, "Yes", "No", KEY_ENTER, KEY_ESC) == 1)
				{
					run = false;
					retVal = -1;
				}
				else
				{
					keyStates.anyKey();
				    timerCounter = 0;
				}
			}

			//Timer stuff
			--timerCounter;
			++frameSkip;
			if(frameSkip > global::MAX_FRAMESKIP || !run)
			{
				timerCounter = 0;
				frameSkip = 0;
				break;
			}
		}
		clear_to_color(global::dbuffer, makecol(0xFD, 0xFA, 0x00));
		blit(background, global::dbuffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		textout_centre_ex(global::dbuffer, font, versus.c_str(), SCREEN_W >> 1, 25, 0, -1);
		messages.draw(262);
		for(i = 0; i < 3; ++i)
		{
			if(units[0].attack[i] >= 0)
			{
				if(buttons[units[0].attack[i]])
					blit(buttons[units[0].attack[i]], global::dbuffer, 0, 0, 20 + 60 * i, 200, buttons[units[0].attack[i]]->w, buttons[units[0].attack[i]]->h);
				textprintf_right_ex(global::dbuffer, font, 70 + 60 * i,
					242, makecol(0xFF, 0xFF, 0xFF), 0, "%i", units[0].attackMana[i]);
			}
		}
		for(i = 0; i < 4; ++i)
		{
			if(buttons[i])
			    blit(buttons[i], global::dbuffer, 0, 0, 570, 200 + i * 60, buttons[i]->w, buttons[i]->h);
		}
		rectfill(global::dbuffer, 20, 155, 20 + int((float(units[0].life) / 100.0) * 130.0), 169, makecol(0x00, 0xFF, 0x00));
		textprintf_centre_ex(global::dbuffer, font, 85, 158, makecol(0xFF, 0xFF, 0xFF), -1, "%i life", units[0].life);
		rectfill(global::dbuffer, 255, 155, 255 + int((float(units[0].mana) / 20.0) * 130.0), 169, makecol(0x00, 0x00, 0xCC));
		textprintf_centre_ex(global::dbuffer, font, 320, 158, makecol(0xFF, 0xFF, 0xFF), -1, "%i mana", units[0].mana);
		rectfill(global::dbuffer, 620, 155, 620 - int((float(units[1].life) / 100.0) * 130.0), 169, makecol(0x00, 0xFF, 0x00));
		textprintf_centre_ex(global::dbuffer, font, 555, 158, makecol(0xFF, 0xFF, 0xFF), -1, "%i life", units[1].life);
		switch(state)
		{
		case SELECTMANA:
			{
				textprintf_centre_ex(global::dbuffer, font, 350, 230, makecol(0xFF, 0xFF, 0xFF), -1, "%i", curMana);
			}
		case SELECTATTACKS:
			{
				circlefill(global::dbuffer, 620, 225 + curFieldSelected * 60, 8, makecol(0xFF, 0x00, 0x00));
				textout_ex(global::dbuffer, font, attackNames[weazards[0].getType()][curFieldSelected].c_str(), 25, 385, 0, -1);
				for(i = 0; i < 3; ++i)
				{
					textout_ex(global::dbuffer, font, description[weazards[0].getType() * 4 + curFieldSelected][i].c_str(), 25, 405 + i * 12, 0, -1);
				}
			}
			break;
		case ANIMATE:
			{
				for(i = 0; i < 2; ++i)
				{
					if(units[i].attack[curAnimation] == 0)
					{
						int posX;
						if(i == 0)
						    posX = 560 - int(float(animCountdown) * 7.5);
						else
						    posX = 80 + int(float(animCountdown) * 7.5);
						circlefill(global::dbuffer, posX, 118, units[i].attackMana[curAnimation] >> 1, colorSpell[weazards[i].getType()]);
					}
					else if(units[i].attack[curAnimation] == 1 && animCountdown < 20)
					{
						int posX = -units[i].attackMana[curAnimation];
						if(i == 0)
						    posX *= -1;
						rectfill(global::dbuffer, 90 + 460 * i, 100, 90 + 460 * i + posX, 140, colorSpell[weazards[i].getType()]);
					}
					else if(units[i].attack[curAnimation] == 2 && animCountdown > 32)
					{
						rectfill(global::dbuffer, 67 + 480 * i, 70, 73 + 480 * i, 100, colorSpell[weazards[i].getType()]);
						rectfill(global::dbuffer, 60 + 480 * i, 77, 80 + 480 * i, 83, colorSpell[weazards[i].getType()]);
					}
					else if(units[i].attack[curAnimation] == 3)
					{
						switch(weazards[i].getType())
						{
						case CT_SPRING:
							int j;
							for(j = 0; j < 5; ++j)
								circlefill(global::dbuffer, 64 + rand() % 18 + i * 480, 108 + rand() % 36 + i * 480, 2, colorSpell[weazards[0].getType()]);
							break;
						case CT_SUMMER:
							{
								int posX;
								if(i == 0)
								    posX = 560 - int(float(animCountdown) * 7.5);
								else
								    posX = 80 + int(float(animCountdown) * 7.5);
								int size1 = 20, size2 = 0;
								if(animCountdown < 48)
							    	size1 -= (size2 = 5);
								circlefill(global::dbuffer, posX, 118, size1, colorSpell[weazards[i].getType()]);
								if(size2 && animCountdown > 32)
								{
									if(i == 1)
									    posX = 440 + int(float(48 - animCountdown) * 7.5);
									else
									    posX = 200 - int(float(48 - animCountdown) * 7.5);
								    circlefill(global::dbuffer, posX, 118, size2, colorSpell[weazards[i].getType()]);
								}
							}
							break;
						case CT_FALL:
							if(animCountdown > 32)
							{
								rectfill(global::dbuffer, 67 + 480 * i, 70, 73 + 480 * i, 100, colorSpell[weazards[i].getType()]);
								rectfill(global::dbuffer, 60 + 480 * i, 77, 80 + 480 * i, 83, colorSpell[weazards[i].getType()]);
							}
							break;
						case CT_WINTER: //71, 126
						    rect(global::dbuffer, 103 - animCountdown, 160 - animCountdown, 39 + animCountdown, 94 + animCountdown, colorSpell[weazards[0].getType()]);
							break;
						};
					}
				}
			}
			break;
		};
		vsync();
		show();
	}
	return retVal;
}

int game(CT character)
{
	unit weazards[2];
	weazards[0].setValues(names[character], character);
	int battlesMastered = 0;
	int i;
	BITMAP *bg = tga_from_char_ex(imgFight);
	if(!bg)
	    error::loadBit("fight.tga");

	BITMAP *buttons[4] = { tga_from_char_ex(imgButtonAttack),
		 tga_from_char_ex(imgButtonDefense),
		 tga_from_char_ex(imgButtonHeal),
		 tga_from_char_ex(imgButtonSpecial) };

	bool run = true;
	int frameSkip = 0;
	while(run)
	{
		CT enemy = static_cast<CT>(rand() & 3);
		if(enemy == weazards[0].getType())
			enemy = static_cast<CT>(rand() & 3);
		weazards[1].setValues(names[enemy], enemy);
		//Announce New battle
		showNames(weazards, battlesMastered + 1);
		/* DONE (Lenman#1#): Show names of the players! */
		//Fight battle
		/* DONE (Lenman#1#): Add code for fighting the fight */
		int winner = fight(weazards, bg, buttons);
		if(winner == 1)
		    ++battlesMastered;
		else
		    run = false;
	}
	destroy_bitmap(bg);
	for(i = 0; i < 4; ++i)
	{
		if(!buttons[i])
			continue;
		destroy_bitmap(buttons[i]);
	}
	return battlesMastered;
}

void showNames(unit *weazards, int battle)
{
	int countdown = 256;
	int frameSkip = 0;
	string text = weazards[0].getName() + string(" (human) vs ") + weazards[1].getName() + string(" (AI)");
	while(countdown)
	{
		while(timerCounter > 0 && countdown)
		{
			if(keyStates.anyKey())
			    countdown = 1;
			//Timer stuff
			--timerCounter;
			--countdown;
			++frameSkip;
			if(frameSkip > global::MAX_FRAMESKIP)
			{
				timerCounter = 0;
				frameSkip = 0;
				break;
			}
		}
		clear_to_color(global::dbuffer, makecol(0xFD, 0xFA, 0x00));
		textout_centre_ex(global::dbuffer, font, (string("Battle ") + i2str(battle)).c_str(), static_cast<int>(float(countdown) * 2.5), (SCREEN_H >> 1) - 10, 0, -1);
		textout_centre_ex(global::dbuffer, font, text.c_str(), static_cast<int>(float(countdown) * 2.5), SCREEN_H >> 1, 0, -1);
		show();
	}
}
