/*
  Name: inlineTGA.h
  Copyright: Master Lenman 2005
  Author: Master Lenman
  Date: 18.06.05 07:47
  Description: A function that will load a bitmap from a file previously
  created with headerTGACreator
*/

#include <allegro.h>
#include "inlineTGA.h"

namespace HeaderTGA
{
	void read(const unsigned char *source, char *dest, int length)
	{
		for(int i = 0; i < length; ++i)
			dest[i] = source[i];
	}

	BITMAP *tga_from_char(const unsigned char *imageData)
	{
		if(!imageData)
			return NULL;

		int offset = 0;
		const int MAXHEADERSIZE = 12;
		const unsigned char PERFECT_HEADER[MAXHEADERSIZE] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int i;

		//Check if file can be used by checking the header
		for(i = 0; i < MAXHEADERSIZE; ++i)
			if(imageData[offset++] != PERFECT_HEADER[i])
			{
				return NULL;
			}

		unsigned short width, height;
		char colorDepth;

		//Read size
		read(imageData + offset, reinterpret_cast<char*>(&width), sizeof(short));
		offset += 2;

		read(imageData + offset, reinterpret_cast<char*>(&height), sizeof(short));
		offset += 2;

		//Read color depth
		read(imageData + offset++, &colorDepth, sizeof(char));
		if(colorDepth < 16)
			return NULL;

		char descriptorByte;
		read(imageData + offset++, &descriptorByte, sizeof(char));
		//allegro_message("%dx%d@%d using %x", width, height, colorDepth, descriptorByte);

		//Check descriptor byte
		switch(colorDepth)
		{
		case 16:
			if(descriptorByte != 0x01)
				return NULL;
			break;
		case 24:
			if(descriptorByte != 0x00)
				return NULL;
			break;
		case 32:
			if(descriptorByte != 0x08)
				return NULL;
			break;
		};

		//create return value bitmap
		BITMAP *retData = create_bitmap_ex(colorDepth, width, height);
		if(!retData)
			return retData;

		//Extract image data
		long buffer = 0;
		for(int y = 0; y < height; ++y)
		{
			for(int x = 0; x < width; ++x)
			{
				read(imageData + offset, reinterpret_cast<char*>(&buffer), colorDepth >> 3);
				offset += colorDepth >> 3;
				switch(colorDepth)
				{
				case 16:
					//Version for higher precision
					//if(highPrecision)
					//	color = makecol16(int(float((buffer >> 10) & 0x1F) * 255.0 / 31.0), int(float((buffer >> 5) & 0x1F) * 255.0 / 31.0), int(float(buffer & 0x1F) * 255.0 / 31.0));
					//else
					//Version if there is no status bit
					//color = makecol16(int(float((buffer >> 11) & 0x1F) * 255.0 / 31.0), int(float((buffer >> 5) & 0x3F) * 255.0 / 63.0), int(float(buffer & 0x1F) * 255.0 / 31.0));
					_putpixel16(retData, x, height - y - 1, makecol16(((buffer >> 10) & 0x1F) << 3, ((buffer >> 5) & 0x1F) << 3, (buffer & 0x1F) << 3));
					break;
				case 24:
					_putpixel24(retData, x, height - y - 1, makecol24((buffer >> 16) & 0xFF, (buffer >> 8) & 0xFF, buffer & 0xFF));
					break;
				case 32:
					_putpixel32(retData, x, height - y - 1, makeacol32((buffer >> 16) & 0xFF, (buffer >> 8) & 0xFF, buffer & 0xFF, (buffer >> 24) & 0xFF));
					break;
				}
			}
		}

		return retData;
	}

	BITMAP *tga_from_char_ex(const unsigned char *imageData)
	{
		BITMAP *retData = tga_from_char(imageData);
		if(retData && bitmap_color_depth(screen) != bitmap_color_depth(retData))
		{
			BITMAP *temp = create_bitmap(retData->w, retData->h);
			//clear_to_color(temp, makecol(0xFF, 0x00, 0xFF));
			//draw_sprite_v_flip(temp, retData, 0, 0);
			blit(retData, temp, 0, 0, 0, 0, retData->w, retData->h);
			destroy_bitmap(retData);
			retData = temp;
		}
		return retData;
	}
}
