/*
  Name: main.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 17.06.05 15:21
  Description: This file contains the core functions of the program:
		main(), init() and uninit() as well as timer routines
*/

#include "StdAfx.h"
#include <ctime>
using std::time;

#include "game.h"
#include "highscore.h"

//Some prototypes
void init();
void uninit();

//The core function of the application
int main(int argc, char **argv)
{
	init();
	CT type;
	while(true)
	{
		type = menu::mainMenu();
		if(type == CT_EXIT)
		    break;
		int score = game(type);
		highscore(score, type);
	}
	/* DONE (Lenman#1#): Add call to game() function */
	uninit();
	return 0;
} END_OF_MAIN()

//timerCounterUpdater() will update timerCounter 32 times per second.
//This timer is used to make the game run at the same speed on all computers.
volatile int timerCounter = 0;
void timerCounterUpdater()
{
	timerCounter++;
} END_OF_STATIC_FUNCTION(timerCounterUpdater)

//initialise allegro and some globals
void init()
{
	//initialise Allegro, timer, keyboard and sound
	allegro_init();

	if(install_timer() < 0)
		error::timer();
	if(install_keyboard() < 0)
		error::keyboard();
	if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) < 0)
		error::soundDriver();

	/* DONE (Any#2#): Paste Game's name */
	//Set a title for the window
	set_window_title("Weazards - Polecatsoft");

	/* DONE (Lenman#1#): Define color depth and resolution */
	set_color_depth(16);
	if(set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) < 0)
	{
		error::graphicMode();
	}

	global::dbuffer = create_bitmap(SCREEN_W, SCREEN_H);
	if(!global::dbuffer)
	    error::create();
	clear(global::dbuffer);
	textout_centre_ex(global::dbuffer, font, "Loading engine, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

    //Lock Timer Function and variable
	LOCK_FUNCTION(timerCounterUpdater);
	LOCK_VARIABLE(timerCounter);

    //Set timer to 32 fps
	install_int_ex(timerCounterUpdater, BPS_TO_TIMER(global::FPS));

	//Set Charset
	set_uformat(U_ASCII);

	//Initialise rand with the time and the randomly picked number 0xABCDE
	srand(time(0) * 0xABCDE);
	
	//Set GUI colors
	gui_bg_color = makecol(0xDD, 0xDD, 0xDD);
}

void uninit()
{
	destroy_bitmap(global::dbuffer);
}

//Display the doublebuffer on the screen
void show()
{
	blit(global::dbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
